/*
  ToDos:
  - Laden anzeigen (neuer Messwert höher als alter)
  
*/
#include <FS.h>                   //this needs to be first, or it all crashes and burns...
#include <ESP8266WiFi.h>
#include "ThingSpeak.h"
#include <OneWire.h>
#include "SSD1306Brzo.h"
#include "OLEDDisplayUi.h"        //in OLEDDisplayUi.h kein private mehr !!!!

#include "Wire.h"
#include <NTPtimeESP.h>
#include "TimeLib.h"
#include "icons.h"
#include <WiFiManager.h>
#include <ArduinoJson.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

unsigned long myPeriodic = 7000;     // in msec | Frame wechsel pub is 05sec
unsigned long myPeriodicTS = 30000;  // in msec | Thingspeak pub is 05sec
unsigned long myPeriodicI2C = 17000; // in msec | DweetIO pub is 10sec
unsigned long myPeriodicNTP = 3600000; // in msec | NTP Update in 1Stunde
unsigned long myPeriodicSensoren = 2000; // in msec | Sensoren Update in 5sec

unsigned long Thimer;
unsigned long Thimer1;
unsigned long Thimer2;
unsigned long Thimer3;
unsigned long Thimer4;
float U_Batt;
float T_UBatt;
int   P_UBatt;

#define ONE_WIRE_BUS 13  // DS18B20 on arduino pin2 corresponds to D4 on physical board
#define TEMPERATURE_PRECISION 11
OneWire Sensoren(ONE_WIRE_BUS);

// arrays to hold device addresses
struct t_arDevAdr {
  byte DevAdr[8];
} arDevAdr[8];
byte DevAdr[8];
byte DevData[12];
float arrTemp[8];
byte DevAnz = 8;

NTPtime NTPch("ch.pool.ntp.org");
strDateTime dateTime;

// Display Settings
const int I2C_DISPLAY_ADDRESS = 0x3c;
const int SDA_PIN = 2;
const int SDC_PIN = 4;

SSD1306Brzo  display(I2C_DISPLAY_ADDRESS, SDA_PIN, SDC_PIN);
OLEDDisplayUi ui     ( &display );

//THINGSPEAK Daten
const char*   server        = "api.thingspeak.com";
char   myWriteAPIKey  [20]  ;//= "VJ2Z73HC48RO38DS";
char SmyChannelNumber [10]  ;//= "222222";
char          mydevId [40]  ;//= "DweetName";
bool TSerr = 0;
bool DWerr = 0;
bool shouldSaveConfig = false;
const char* host            = "www.dweet.io";
const int   httpPort        = 80;

WiFiManagerParameter custom_myWriteAPIKey    ("API-Key", "myWriteAPIKey", myWriteAPIKey, 20);
WiFiManagerParameter custom_SmyChannelNumber ("ChannelNumber", "SmyChannelNumber", SmyChannelNumber, 10);
WiFiManagerParameter custom_mydevId            ("DevId", "devId", mydevId, 40);

float temp0;
float temp1;
float temp2;
float temp3;
float temp4;
float temp5;
float temp6;
float temp7;
WiFiServer WifiServer(80);
WiFiClient client;
//***************************************************************************************
// Connect Bildschirm einblenden
//***************************************************************************************
void ConSchirm(void) {
  display.clear();
  display.setFont(ArialMT_Plain_10);
  display.drawString(64, 10, "Connecting to WiFi");
  display.drawXbm(46, 30, 8, 8, millis() / 500 % 3 == 0 ? activeSymbol : inactiveSymbol);
  display.drawXbm(60, 30, 8, 8, millis() / 500 % 3 == 1 ? activeSymbol : inactiveSymbol);
  display.drawXbm(74, 30, 8, 8, millis() / 500 % 3 == 2 ? activeSymbol : inactiveSymbol);
  display.display();
}
//***************************************************************************************
// Connect Bildschirm einblenden
//***************************************************************************************
void NTPBildschirm () {
  display.clear();
  display.setFont(ArialMT_Plain_10);
  display.drawString(64, 10, "NTP-Server Abfrage");
  display.drawString(64, 30, "..................");
  display.display();
}
//***************************************************************************************
// Meldung am TFT wenn Configmode
//***************************************************************************************
//gets called before WiFiManager enters configuration mode
void configModeCallback (WiFiManager * myWiFiManager) {
  WiFi.disconnect();
  display.clear();
  display.setFont(ArialMT_Plain_10);
  display.drawString(64, 5,  "Wifi ned gfundn !!!");
  display.drawString(64, 20, "Im Browser: 192.168.4.1");
  display.drawString(64, 37, "Config-Access-Point:");
  display.drawString(64, 50, String(myWiFiManager->getConfigPortalSSID()));
  display.display();
}
//***************************************************************************************
// Connect Bildschirm einblenden
//***************************************************************************************
void ConnectBildschirm () {
  display.clear();
  display.setFont(ArialMT_Plain_10);
  display.drawString(64, 10, "Connected to WiFi");
  display.drawString(64, 30, "IP Adresse:");
  IPAddress myIP = WiFi.localIP();
  String ipStr = String(myIP[0]) + "." + String(myIP[1]) + "." + String(myIP[2]) + "." + String(myIP[3]);
  display.drawString(64, 50, ipStr );
  display.display();
  delay(2000);
}
//***************************************************************************************
// Start Bildschirm einblenden
//***************************************************************************************
void StartBildschirm () {
  display.init();
  display.clear();
  display.display();
  //display.flipScreenVertically();
  display.setFont(ArialMT_Plain_16);
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.setContrast(255);
  display.clear();
  display.drawString(64, 10, "Heiz-Messer");
  display.drawString(64, 40, "Version V01l");
  display.display();
  delay (2300);
}
//***************************************************************************************
// Start Bildschirm einblenden
//***************************************************************************************
void ParameterBildschirm () {
  display.init();
  display.clear();
  display.display();
  //display.flipScreenVertically();
  display.setFont(ArialMT_Plain_10);
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.setContrast(255);
  display.clear();
  display.drawString(64, 5, myWriteAPIKey);
  display.setFont(ArialMT_Plain_16);
  display.drawString(64, 20, SmyChannelNumber);
  display.drawString(64, 37, mydevId);
  display.display();
  delay (2300);
}
//***************************************************************************************
// Neue Parameter nach Config Mode sichern
//***************************************************************************************
//callback notifying us of the need to save config
void saveConfigCallback () {
  Serial.println("Should save config");
  shouldSaveConfig = true;
}
//***************************************************************************************
// float Temperatur auf TFT ausgeben
//***************************************************************************************
void Print_Temp(float zahl, int x, int y) {
  char text [55];
  dtostrf(zahl, 3, 1, text);
  display.drawString(x, y, text);
}
//***************************************************************************************
// Uhrzeit zusammenbaun            n
//***************************************************************************************
String Zeit() {
  time_t t = now();
  String Z = "";
  int x = hour(t);
  if (x < 10) {
    Z += "0";
  }
  Z += String(x);
  Z += ":";
  x = minute(t);
  if (x < 10) {
    Z += "0";
  }
  Z += String(x);
  //Z += ":";
  return Z;
}
//***************************************************************************************
// Frames zusammenbauen
//***************************************************************************************
void drawFrame1(OLEDDisplay * display, OLEDDisplayUiState * state, int16_t x, int16_t y) {
  char text [55];
  //display->flipScreenVertically();
  display->setTextAlignment(TEXT_ALIGN_LEFT);
  display->setFont(ArialMT_Plain_16);
  if (WiFi.status() != WL_CONNECTED) {
    display->drawString( x + 23, y + 0, "Kein WiFi !");
  } else {
    display->setFont(ArialMT_Plain_10);
    display->drawString(x + 3, y + 2, "Temp 1-4");
    //ERR Anzeige
    if (TSerr == 1) {
      display->drawString(x + 49, y + 4, "t");
    } else {
      display->drawString(x + 49, y + 4, " ");
    }
    if (DWerr == 1) {
      display->drawString(x + 47, y + 4, "d");
    } else {
      display->drawString(x + 47, y + 4, " ");
    }
    //Zeit anzeigen
    display->setFont(ArialMT_Plain_16);
    display->drawString(x + 52, y + 0, Zeit());
    //U_Batt anzeigen
    if (U_Batt < 4.3) {
      display->drawHorizontalLine(x + 99, y + 3, 22);
      display->drawHorizontalLine(x + 99, y + 14, 22);
      display->drawVerticalLine  (x + 96, y + 5, 8);
      display->drawVerticalLine  (x + 97, y + 5, 8);
      display->drawVerticalLine  (x + 98, y + 4, 10);
      display->drawVerticalLine  (x + 121, y + 4, 10);
      display->fillRect          (x + 100 + (100 - P_UBatt) / 5, y + 4, x + 21 - (100 - P_UBatt) / 5, 10);
    } else {
      display->drawString( x + 95, y + 0, "Voll");
    }
  }
  //Temps anzeigen
  display->setFont(ArialMT_Plain_24);
  if ((float) arrTemp[0] > 2000) {
    display->drawString( x + 3, y + 17, " k.S");
  } else {
    dtostrf(arrTemp[0], 3, 1, text);
    display->drawString( x + 3, y + 17, text);
  }
  if ((float) arrTemp[1] > 2000) {
    display->drawString( x + 65, y + 17, " k.S");
  } else {
    dtostrf(arrTemp[1], 3, 1, text);
    display->drawString( x + 65, y + 17, text);
  }
  if ((float) arrTemp[2] > 2000) {
    display->drawString( x + 3, y + 38, " k.S");
  } else {
    dtostrf(arrTemp[2], 3, 1, text);
    display->drawString( x + 3, y + 38, text);
  }
  if ((float) arrTemp[3] > 2000) {
    display->drawString( x + 65, y + 38, " k.S");
  } else {
    dtostrf(arrTemp[3], 3, 1, text);
    display->drawString( x + 65, y + 38, text);
  }
  display->drawHorizontalLine(x + 0, y + 17, 128);
  display->drawVerticalLine(x + 63, y + 18, 45);
  display->drawRect(x + 0  , y + 0, 128, 63);
}
void drawFrame2(OLEDDisplay * display, OLEDDisplayUiState * state, int16_t x, int16_t y) {
  char text [55];
  //display->flipScreenVertically();
  display->setTextAlignment(TEXT_ALIGN_LEFT);
  display->setFont(ArialMT_Plain_16);
  if (WiFi.status() != WL_CONNECTED) {
    display->drawString( x + 23, y + 0, "Kein WiFi !");
  } else {

    display->setFont(ArialMT_Plain_10);
    display->drawString(x + 3, y + 2, "Temp 5-8");
    //ERR Anzeige
    if (TSerr == 1) {
      display->drawString(x + 49, y + 4, "t");
    } else {
      display->drawString(x + 49, y + 4, " ");
    }
    if (DWerr == 1) {
      display->drawString(x + 47, y + 4, "d");
    } else {
      display->drawString(x + 47, y + 4, " ");
    }
    //Zeit anzeigen
    display->setFont(ArialMT_Plain_16);
    display->drawString(x + 52, y + 0, Zeit() );
    //U_Batt anzeigen
    if (U_Batt < 4.3) {
      display->drawHorizontalLine(x + 99, y + 3, 22);
      display->drawHorizontalLine(x + 99, y + 14, 22);
      display->drawVerticalLine(x + 96, y + 5, 8);
      display->drawVerticalLine(x + 97, y + 5, 8);
      display->drawVerticalLine(x + 98, y + 4, 10);
      display->drawVerticalLine(x + 121, y + 4, 10);
      display->fillRect(x + 100 + (100 - P_UBatt) / 5, y + 4, x + 21 - (100 - P_UBatt) / 5, 10);
    } else {
      display->drawString( x + 95, y + 0, "Voll");
    }
  }
  //Temps anzeigen
  display->setFont(ArialMT_Plain_24);
  if ((float) arrTemp[4] > 2000) {
    display->drawString( x + 3, y + 17, " k.S");
  } else {
    dtostrf(arrTemp[4], 3, 1, text);
    display->drawString( x + 3, y + 17, text);
  }
  if ((float) arrTemp[5] > 2000) {
    display->drawString( x + 65, y + 17, " k.S");
  } else {
    dtostrf(arrTemp[5], 3, 1, text);
    display->drawString( x + 65, y + 17, text);
  }
  if ((float) arrTemp[6] > 2000) {
    display->drawString( x + 3, y + 38, " k.S");
  } else {
    dtostrf(arrTemp[6], 3, 1, text);
    display->drawString( x + 3, y + 38, text);
  }
  if ((float) arrTemp[7] > 2000) {
    display->drawString( x + 65, y + 38, " k.S");
  } else {
    dtostrf(arrTemp[7], 3, 1, text);
    display->drawString( x + 65, y + 38, text);
  }
  display->drawHorizontalLine(x + 0, y + 17, 128);
  display->drawVerticalLine(x + 63, y + 18, 45);
  display->drawRect(x + 0  , y + 0, 128, 63);
}

// this array keeps function pointers to all frames
// frames are the single views that slide from right to left
FrameCallback frames[] = { drawFrame1, drawFrame2}; //, drawFrame3, drawFrame4};
int numberOfFrames = 2;
//***************************************************************************************
// Init SPIFFS "Festplatte" Einrichten
//***************************************************************************************
void Init_SPIFFS() {
  //clean FS, for testing
  //SPIFFS.format();

  //read configuration from FS json
  Serial.println("mounting FS...");

  if (SPIFFS.begin()) {
    Serial.println("mounted file system");
    if (SPIFFS.exists("/config.json")) {
      //file exists, reading and loading
      Serial.println("reading config file");
      File configFile = SPIFFS.open("/config.json", "r");
      if (configFile) {
        Serial.println("opened config file");
        size_t size = configFile.size();
        // Allocate a buffer to store contents of the file.
        std::unique_ptr<char[]> buf(new char[size]);

        configFile.readBytes(buf.get(), size);
        DynamicJsonBuffer jsonBuffer;
        JsonObject& json = jsonBuffer.parseObject(buf.get());
        json.printTo(Serial);
        if (json.success()) {
          Serial.println("\nparsed json");

          strcpy(myWriteAPIKey, json["myWriteAPIKey"]);
          strcpy(SmyChannelNumber, json["SmyChannelNumber"]);
          strcpy(mydevId, json["devId"]);
          ParameterBildschirm ();
        } else {
          Serial.println("failed to load json config");
        }
      }
    }
  } else {
    Serial.println("failed to mount FS");
  }
  //end read

  // The extra parameters to be configured (can be either global or just in the setup)
  // After connecting, parameter.getValue() will get you the configured value
  // id/name placeholder/prompt default length
}
//***************************************************************************************
// Wifi Manager Initialisieren
//***************************************************************************************
void InitWiFiManager() {
  WiFiManager wifiManager;
  WiFi.setAutoConnect(true);
  wifiManager.setSaveConfigCallback(saveConfigCallback);
  wifiManager.setBreakAfterConfig(true);
  wifiManager.setAPCallback(configModeCallback);
  wifiManager.addParameter(&custom_myWriteAPIKey);
  wifiManager.addParameter(&custom_SmyChannelNumber);
  wifiManager.addParameter(&custom_mydevId);
  wifiManager.setConfigPortalTimeout(100);
  wifiManager.setDebugOutput(true);
  //reset settings - for testing
  //wifiManager.resetSettings();
  //fetches ssid and pass and tries to connect
  //if it does not connect it starts an access point with the specified name
  //here  "AutoConnectAP"
  //and goes into a blocking loop awaiting configuration
  if (!wifiManager.autoConnect("HeizMessConfig")) {
    for (byte i = 0; i < 3; i++) {
      display.clear();
      display.drawString(54, 10, "Niemand verbunden");
      String text = "Neustart in: " + String (3 - i) + "s";
      display.drawString(54, 30, text);
      display.display();
      delay(1000);
    }
    WiFi.softAPdisconnect(true); //AP Ausschalten
    //reset and try again, or maybe put it to deep sleep
    ESP.reset();
    delay(1000);
  }
  WiFi.softAPdisconnect(true); //AP Ausschalten
}
//***************************************************************************************
// UI-Bildschirmframes Initialisieren, iniztialisiert auch Display
//***************************************************************************************
void InitUI() {
  ui.setTargetFPS(20);
  ui.disableAllIndicators();
  ui.disableAutoTransition();
  // You can change the transition that is used
  // SLIDE_LEFT, SLIDE_RIGHT, SLIDE_TOP, SLIDE_DOWN
  ui.setFrameAnimation(SLIDE_LEFT);
  // Add frames
  ui.setFrames(frames, numberOfFrames);
  // Inital UI takes care of initalising the display too.
  ui.init();
}
//***************************************************************************************
// Arduino OTA Initialisierung
//***************************************************************************************
void InitOTA() {
  ArduinoOTA.onStart([]() {
    Serial.println("Start");
  });
  ArduinoOTA.onEnd([]() {
    Serial.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
    else if (error == OTA_END_ERROR) Serial.println("End Failed");
  });
  ArduinoOTA.begin();
  Serial.println("OTA Ready");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}
//***************************************************************************************
// Config Portal Parameter Speichern
//***************************************************************************************
void Save_CP_Parameter() {
  //save the custom parameters to FS
  if (custom_myWriteAPIKey.getValue() == "") {
    return;
  }
  if (custom_SmyChannelNumber.getValue() == "") {
    return;
  }
  if (custom_mydevId.getValue() == "") {
    return;
  }
  if (shouldSaveConfig) {
    Serial.println("Dadn gschbaichad");
    strcpy(myWriteAPIKey, custom_myWriteAPIKey.getValue());
    strcpy(SmyChannelNumber, custom_SmyChannelNumber.getValue());
    strcpy(mydevId, custom_mydevId.getValue());
    DynamicJsonBuffer jsonBuffer;
    JsonObject& json = jsonBuffer.createObject();
    json["myWriteAPIKey"] = myWriteAPIKey;
    json["SmyChannelNumber"] = SmyChannelNumber;
    json["devId"] = mydevId;

    File configFile = SPIFFS.open("/config.json", "w");
    if (!configFile) {
      Serial.println("failed to open config file for writing");
    }
    json.printTo(Serial);
    json.printTo(configFile);
    configFile.close();
    ParameterBildschirm ();
    //end save
  }
}
//***************************************************************************************
// Setup
//***************************************************************************************
void setup() {
  Serial.begin(115200);
  pinMode(A0, INPUT);
  ThingSpeak.begin(client);
  StartBildschirm ();
  Init_SPIFFS();
  ConSchirm();
  InitWiFiManager();
  Save_CP_Parameter();
  InitUI();
  InitOTA();
  Thimer   = millis() ;
  Thimer1  = millis() ;
  Thimer2  = millis() ;
  Thimer3  = millis() ;
  Thimer4  = millis() ;
  NTPBildschirm ();
  dateTime = NTPch.getNTPtime(1.0, 1);
  setTime(dateTime.hour, dateTime.minute, dateTime.second,
          dateTime.day, dateTime.month, dateTime.year);
  ConnectBildschirm();

}
void loop() {
  ArduinoOTA.handle();
  //Anzeigen Timer
  if ( (unsigned long) (millis() - Thimer) >= myPeriodic) {
    Thimer = millis();
    ui.nextFrame();
  }
  //Messen und Verschicken nur wenn Bildschirm ned schiebt
  if (ui.state.frameState != IN_TRANSITION) {
    if ( (unsigned long) (millis() - Thimer4) >= myPeriodicSensoren) {
      Thimer4 = millis();
      //Akku Spannung auslesen
      U_Batt = (float) analogRead(A0) * 0.006766;
      if (U_Batt < 3.2) {
        T_UBatt = 3.2;
      } else {
        T_UBatt = U_Batt;
      }
      P_UBatt = (int)((T_UBatt - 3.2) * 100);
      //Serial.println(P_UBatt);
      //Temperaturen auslesen:
      //Serial.print("Sensoren: ");
      for (byte i = 0; i < DevAnz; i++) {
        if (!Sensoren.search(DevAdr)) {
          DevAnz = i + 1;
        }
        Sensoren.reset();
        Sensoren.select(DevAdr);
        Sensoren.write(0x44, 1);        // start conversion, with parasite power on at the end
        for (byte k = 0; k < 8; k++) {
          arDevAdr[i].DevAdr[k] = DevAdr[k];
        }
      }
      delay(350);     // maybe 750ms is enough, maybe not
      for (byte i = 0; i < DevAnz; i++) {
        for (byte k = 0; k < 8; k++) {
          DevAdr[k] = arDevAdr[i].DevAdr[k];
        }
        Sensoren.reset();
        Sensoren.select(DevAdr);
        Sensoren.write(0xBE);         // Read Scratchpad
        for (byte j = 0; j < 9; j++) {           // we need 9 bytes
          DevData[j] = Sensoren.read();
          //Serial.print(i);
          //Serial.print(" ");
        }
        //Sensor-Adresse auslesen
        byte SensorNr = DevData[4] & 0xF;
        // Convert the data to actual temperature
        // because the result is a 16 bit signed integer, it should
        // be stored to an "int16_t" type, which is always 16 bits
        // even when compiled on a 32 bit processor.
        int16_t raw = (DevData[1] << 8) | DevData[0];
        byte cfg = (DevData[4] & 0x60);
        // at lower res, the low bits are undefined, so let's zero them
        if (cfg == 0x00) raw = raw & ~7;  // 9 bit resolution, 93.75 ms
        else if (cfg == 0x20) raw = raw & ~3; // 10 bit res, 187.5 ms
        else if (cfg == 0x40) raw = raw & ~1; // 11 bit res, 375 ms
        //// default is 12 bit resolution, 750 ms conversion time
        arrTemp[SensorNr] = (float)raw / 16.0;
      }
      Sensoren.reset_search();
      //arrTemp[1] = U_Batt;
      //Serial.println("!");
    }
    //Thingspeak Timer
    if ( (unsigned long) (millis() - Thimer1) >= myPeriodicTS) {
      Thimer1 = millis();
      //Thingspeak vorbereiten:
      for (byte i = 0; i < 8; i++) {
        ThingSpeak.setField(i + 1, arrTemp[i]);
      }
      // Daten wegschicken
      ThingSpeak.writeFields(String(SmyChannelNumber).toInt(), myWriteAPIKey);
      //Fehler anzeigen funkt ned wenn Wifi weg is !
      int resultCode = ThingSpeak.getLastReadStatus();
      if (resultCode != 200)
      {
        Serial.print("TS err: ");
        Serial.println(resultCode);
        TSerr = 1;
      }
      else
      {
        Serial.println("TS ok");
        resultCode = 0;
        TSerr = 0;
      }

    }
    // DWEET.IO Timer :
    if ( (unsigned long) (millis() - Thimer2) >= myPeriodicI2C) {
      Thimer2 = millis();
      if (!client.connect(host, httpPort)) {
        Serial.println("Dweet connection failed");
        return;
      }
      // Daten zusammenbasteln
      String url = String(mydevId) + "?";
      url += "TEMP0=" + String(arrTemp[0]);
      for (byte i = 1; i < 8; i++) {
        url += "&TEMP" + String(i) + "=" + String(arrTemp[i]);
      }
      url += "&BATT=" + String(U_Batt);
      url += "&TStamp=" + String(now());
      // This will send the request to the server
      client.print("GET /dweet/for/" + url + " HTTP/1.1\r\n" +
                   "Host: " + host + "\r\n" +
                   "Connection: close\r\n\r\n");
      delay(550);
      String line;
      while (client.available()) {
        line = client.readStringUntil('\r');
      }
      if (line.substring(10, 18) == "succeeded") {
        Serial.println("Dweet err");
        DWerr = 1;
      } else {
        Serial.println("Dweet ok");
        DWerr = 0;
      }
      client.stop();
    }
    // NTP-Timer lesen und die interne Uhr setzen
    if ( (unsigned long) (millis() - Thimer3) >= myPeriodicNTP) {
      Thimer3 = millis();
      dateTime = NTPch.getNTPtime(1.0, 1);
      setTime(dateTime.hour, dateTime.minute, dateTime.second,
              dateTime.day, dateTime.month, dateTime.year);
    }
  }
  ui.update();
}


